#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: backup_restore.pl, v2.0.0-5 Tue 29 Nov 2011 09:44:51 PM CET mstauber Exp $
# Copyright 2006-2011 Stauber Multimedia Design. All rights reserved.

# This handler is triggered whenever a user initiates a restore a VPS backup
# through the GUI.

$whatami = "handler";

use CCE;
my $cce = new CCE('Domain' => 'base-vserver');
use POSIX qw(isalpha);
use Socket;
use Sys::Hostname;
use File::Spec;
use List::MoreUtils qw(uniq);
use Net::SFTP::Foreign;
use File::Basename;
use POSIX qw(strftime);
use DateTime;

# Helper to redirect SSH errors to /dev/null:
open $ssherr, '>', '/dev/null'; # or die "Unable to open /dev/null";

use File::Remote qw(:replace);       # special :replace tag
setrsh("/usr/bin/ssh");
setrcp("/usr/bin/scp");

# Debugging switch:
$DEBUG = "0";
if ($DEBUG) {
    use Sys::Syslog qw( :DEFAULT setlogsock);
}

if ($whatami eq "handler") {
    $cce->connectfd();
}
else {
    $cce->connectuds();
}

# If this is a secondary in a cluster, we can save us the bother:
if (!-d "/vz/private") {
	$cce->bye('SUCCESS');
	exit 0;
}

$event = $cce->event_object();
$event_new = $cce->event_new();
$event_old = $cce->event_old();

&debug_msg("INFO: Passed cluster check \n");
&debug_msg("INFO: event: $event->{'OID'} \n");
&debug_msg("INFO: event_new: $event_new->{'OID'} \n");
&debug_msg("INFO: event_old: $event_old->{'OID'} \n");

# Get settings out of CODB to determine the target of the remote backup (if any):
@oids = $cce->find('VserverMasterNode');
($ok, $solbackupValues) = $cce->get($oids[0]);
if ($ok) {
    # Define values:
    $vzdump_enabled		= $solbackupValues->{'vzdump_enabled'};
    $rsync_enabled		= $solbackupValues->{'rsync_enabled'};
    $MN_RemoteBackupEnabled	= $solbackupValues->{'MN_RemoteBackupEnabled'};
    $MN_LocalBackupEnabled	= $solbackupValues->{'MN_LocalBackupEnabled'};
    $MN_BackupLocalLocation	= $solbackupValues->{'MN_BackupLocalLocation'};
    $targethost                 = $solbackupValues->{'MN_BackupRemoteTargetHost'};
    $targetip                   = $solbackupValues->{'MN_BackupRemoteTargetIP'};
    $path                       = $solbackupValues->{'MN_BackupRemoteLocation'};
    $user                       = $solbackupValues->{'MN_BackupRemoteTargetUser'};
    $pass                       = $solbackupValues->{'MN_BackupRemoteTargetUserPass'};
    $port                       = $solbackupValues->{'MN_BackupRemoteTargetPort'};
    $limit                      = $solbackupValues->{'RsyncSpeedLimit'};
    $my_maxfiles                = $solbackupValues->{'vzdump_maxfiles'};
    $kvm_support                = $solbackupValues->{'kvm_support'};

    # Assemble paths:
    $local_location_vzdump = $solbackupValues->{'MN_BackupLocalLocation'} . "/vzbackup";
    $local_location_vzdump =~ s|//|/|g;
    $local_location_vzdump =~ s|/$||g;

    $remote_location_vzdump = $solbackupValues->{'MN_BackupRemoteLocation'};
    $remote_location_vzdump =~ s|//|/|g;
    $remote_location_vzdump =~ s|/$||g;

    $local_location_rsync = $solbackupValues->{'MN_BackupLocalLocation'} . "/rsync";
    $local_location_rsync =~ s|//|/|g;
    $local_location_rsync =~ s|/$||g;

    $remote_location_rsync = $solbackupValues->{'MN_BackupRemoteLocation'};
    $remote_location_rsync =~ s|//|/|g; 
    $remote_location_rsync =~ s|/$||g; 
}

# Get the data of the Backup Object we want to delete:
($ok, $Backup) = $cce->get($event->{'OID'});
if ($ok) {
    if (($Backup->{'location'} eq "local") && ($Backup->{'origin'} eq "vzdump") && (-f $Backup->{'storage'}) && ($Backup->{'targetvpsid'})) {
	if (($Backup->{'virttype'} eq "openvz") && (-d "/vz/private/$Backup->{'targetvpsid'}")) {
	    &debug_msg("INFO: Stopping old VPS $Backup->{'targetvpsid'} in preparation for the restore. \n");
	    system("/usr/sbin/vzctl stop $Backup->{'targetvpsid'}");
	    &debug_msg("INFO: Destroying old VPS $Backup->{'targetvpsid'} in preparation for the restore. \n");
	    system("/usr/sbin/vzctl destroy $Backup->{'targetvpsid'}");
	}
	&debug_msg("INFO: OID $event->{'OID'} containing a local vzdump backup at $Backup->{'storage'} is going to be restored. \n");
	system("/usr/sbin/vzrestore --force $Backup->{'storage'} $Backup->{'targetvpsid'}");
	&debug_msg("INFO: Restore of VPS $Backup->{'targetvpsid'} from local vzdump backup at $Backup->{'storage'} has finished. \n");
	&debug_msg("INFO: Starting VPS $Backup->{'targetvpsid'} again after the restore. \n");
	system("/usr/sbin/vzctl start $Backup->{'targetvpsid'}");
	&debug_msg("INFO: Restore of VPS $Backup->{'targetvpsid'} has finished. VPS is starting now ... \n");
    }
    if (($Backup->{'location'} eq "local") && ($Backup->{'origin'} eq "rsync") && (-d $Backup->{'storage'}) && ($Backup->{'targetvpsid'})) {
	$vpsid = $Backup->{'vpsid'};
	if ($Backup->{'virttype'} eq "openvz") {
	    &debug_msg("INFO: OID $event->{'OID'} containing a local OpenVZ rsync backup ($vpsid) at $Backup->{'storage'} is going to be restored. \n");
	    # Check if we have the prerequisites for a restore: The directory containing the RSYNC data of the VPS and a VPS config file:
	    if ((-d "$Backup->{'storage'}" ) && (-e "$Backup->{'storage'}/etc/vzdump/vps.conf" )) {
		$vps_config_file = "$Backup->{'storage'}/etc/vzdump/vps.conf";
		&debug_msg("INFO: Found $Backup->{'storage'} containing a local rsync backup and its config file $vps_config_file. \n");
    		if (($Backup->{'virttype'} eq "openvz") && (-d "/vz/private/$Backup->{'targetvpsid'}")) {
        	    &debug_msg("INFO: Stopping old VPS $Backup->{'targetvpsid'} in preparation for the restore. \n");
        	    system("/usr/sbin/vzctl stop $Backup->{'targetvpsid'}");
    		    &debug_msg("INFO: Destroying old VPS $Backup->{'targetvpsid'} in preparation for the restore. \n");
        	    system("/usr/sbin/vzctl destroy $Backup->{'targetvpsid'}");
    		}
        	# Copy the data from the backup location to the production environment:
        	&debug_msg("INFO: Starting local RSYNC from $Backup->{'storage'}/ to /vz/private/$Backup->{'targetvpsid'}/ to restore VPS $Backup->{'targetvpsid'} \n");
        	system("/usr/bin/rsync -q -ar $Backup->{'storage'}/ /vz/private/$Backup->{'targetvpsid'}/ --delete-after");
        	&debug_msg("INFO: Copying VPS config from $vps_config_file to /etc/vz/conf/$Backup->{'targetvpsid'}.conf \n");
        	system("/bin/cp $vps_config_file /etc/vz/conf/$Backup->{'targetvpsid'}.conf");
		system("/bin/chmod 644 /etc/vz/conf/$Backup->{'targetvpsid'}.conf");
        	# Start the VPS again:
        	&debug_msg("INFO: Restore of VPS $Backup->{'targetvpsid'} finished. Starting the restored VPS now. \n");
        	system("/usr/sbin/vzctl start $Backup->{'targetvpsid'}");
    	    }
	}
	elsif ($Backup->{'virttype'} eq "kvm") {
	    &debug_msg("INFO: OID $event->{'OID'} containing a local KVM rsync backup ($vpsid) at $Backup->{'storage'} is going to be restored. \n");
	    # Check if we have the prerequisites for a restore: The directory should contain a disk image and an XML file:
	    if ((-d "$Backup->{'storage'}" ) && (-e "$Backup->{'storage'}/$vpsid.img") && (-e "$Backup->{'storage'}/$vpsid.xml")) {
		$vps_config_file = "$Backup->{'storage'}/$vpsid.xml";
		&debug_msg("INFO: Found $Backup->{'storage'} containing a local KVM ($vpsid) rsync backup and its config file $vps_config_file. \n");
    		if (($Backup->{'virttype'} eq "kvm") && (-f "/vz/kvm/images/$Backup->{'targetvpsid'}.img")) {
        	    &debug_msg("INFO: Stopping old KVM $Backup->{'targetvpsid'} in preparation for the restore. \n");
        	    system("/usr/bin/virsh destroy $Backup->{'targetvpsid'}");
    		    &debug_msg("INFO: Destroying old KVM $Backup->{'targetvpsid'} in preparation for the restore. \n");
        	    system("/usr/bin/virsh undefine $Backup->{'targetvpsid'}");
    		}
        	# Copy the data from the backup location to the production environment:
        	&debug_msg("INFO: Starting local RSYNC from $Backup->{'storage'}//$vpsid.img to /vz/kvm/images/$Backup->{'targetvpsid'}.img to restore KVM $Backup->{'targetvpsid'} disk. \n");
        	system("/usr/bin/rsync -q -ar $Backup->{'storage'}/$vpsid.img /vz/kvm/images/$Backup->{'targetvpsid'}.img --delete-after");
        	system("chown qemu:qemu /vz/kvm/images/$Backup->{'targetvpsid'}.img");
		system("/bin/chmod 600 /vz/kvm/images/$Backup->{'targetvpsid'}.img");
        	&debug_msg("INFO: Copying VPS config from $vps_config_file to /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml \n");
        	system("/bin/cp $vps_config_file /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml");
		system("/bin/chmod 600 /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml");
		system("chown root:root /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml");
		# Define the KVM again:
        	&debug_msg("INFO: Defining KVM $Backup->{'targetvpsid'} from /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml \n");
        	system("/usr/bin/virsh define /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml");
        	# Start the KVM again:
        	&debug_msg("INFO: Restore of KVM $Backup->{'targetvpsid'} finished. Starting the restored KVM now. \n");
        	system("/usr/bin/virsh start $Backup->{'targetvpsid'}");
    	    }
	}
	else {
	    # This is not supposed to happen:
	    &debug_msg("WARN: OID $event->{'OID'} containing an *unknown* type of VPS backup ($vpsid) at $Backup->{'storage'} is NOT going to be restored. \n");
	}
    }
    if (($Backup->{'location'} eq "remote") && ($Backup->{'origin'} eq "vzdump")  && ($Backup->{'targetvpsid'})) {
	&debug_msg("INFO: OID $event->{'OID'} containing a remote vzdump backup at $Backup->{'storage'} is going to be restored. \n");
	if ($user && $pass && $port && ($targethost||$targetip) && $remote_location_vzdump) {
	    if ($targethost) {
    		$target = $user . '@' . $targethost;
	    }
	    else {   
    		$target = $user . '@' . $targetip;
	    }
	    # Connect to remote storage:
	    no warnings 'Net::SFTP::Foreign';
	    $sftp = Net::SFTP::Foreign->new($target, password => $pass, port => $port, autodie => 0, stderr_fh => $ssherr);
	    $sftp->die_on_error("Unable to connect to remote host");
	    &debug_msg("INFO: Connected to $target at port $port \n");
            # Does the file exist?
            $fh1 = $sftp->stat("$Backup->{'storage'}");
            if ($fh1) {
        	&debug_msg("INFO: File $Backup->{'storage'} exists on remote storage. \n");
		# Make sure the target directory /vz/vztmp/ exists first:
		if (-d "/vz/vztmp/") {
		    system("mkdir '/vz/vztmp/'");
		}
    		# Get the filename of the tarball without the local parts of the path:
    		$base_name = basename($Backup->{'storage'});
		# Make sure the temp directory /vz/vztmp/ doesn't have a previous copy in it:
		if (-f "/vz/vztmp/$base_name") {
		    &debug_msg("INFO: Tarball /vz/vztmp/$base_name exists already. Deleting it. \n");
		    $hauweg = "/vz/vztmp/$base_name";
		    system("/bin/rm -f $hauweg");
		}
		# Retrieve the remote backup file via SFTP
		$fh2 = $sftp->mget("$Backup->{'storage'}", "/vz/vztmp/");
                # Start restore after tarball retrieval:
                if ($fh2) {
    		    &debug_msg("INFO: Retrieved $Backup->{'storage'} as requested and stored it as /vz/vztmp/$base_name \n");
		    if (($Backup->{'virttype'} eq "openvz") && (-d "/vz/private/$Backup->{'targetvpsid'}")) {
			&debug_msg("INFO: Stopping old VPS $Backup->{'targetvpsid'} in preparation for the restore. \n");
			system("/usr/sbin/vzctl stop $Backup->{'targetvpsid'}");
			&debug_msg("INFO: Destroying old VPS $Backup->{'targetvpsid'} in preparation for the restore. \n");
			system("/usr/sbin/vzctl destroy $Backup->{'targetvpsid'}");
		    }
		    &debug_msg("INFO: Starting restore of retrieved remote backup $Backup->{'storage'} from temp file /vz/vztmp/$base_name \n");
		    system("/usr/sbin/vzrestore --force /vz/vztmp/$base_name $Backup->{'targetvpsid'}");
		    &debug_msg("INFO: Restore of VPS $Backup->{'targetvpsid'} from remote vzdump backup at $Backup->{'storage'} has finished. \n");
		    if (-f "/vz/vztmp/$base_name") {
			&debug_msg("INFO: Deleting temporary file /vz/vztmp/$base_name.\n");
			$hauweg = "/vz/vztmp/$base_name";
			system("/bin/rm -f $hauweg");
		    }
		    &debug_msg("INFO: Starting VPS $Backup->{'targetvpsid'} again after the restore. \n");
		    system("/usr/sbin/vzctl start $Backup->{'targetvpsid'}");
		    &debug_msg("INFO: Restore of VPS $Backup->{'targetvpsid'} has finished. VPS is starting now ... \n");
                }
                else {
		    &debug_msg("INFO: Restore of VPS $Backup->{'targetvpsid'} failed. Retrieval of $Backup->{'storage'} failed. \n");
                }
            }
	}
    }
    if (($Backup->{'location'} eq "remote") && ($Backup->{'origin'} eq "rsync") && ($Backup->{'targetvpsid'})) {
	$vpsid = $Backup->{'vpsid'};
	# Handle remote restore of OpenVZ VPS's:
	if ($Backup->{'virttype'} eq "openvz") {
	    &debug_msg("INFO: OID $event->{'OID'} containing a remote rsync backup at $Backup->{'storage'} is going to be restored. \n");
	    if ($user && $pass && $port && ($targethost||$targetip) && $remote_location_vzdump) {
		if ($targethost) {
    		    $target = $user . '@' . $targethost;
		    $target_short = $targethost;
		}
		else {   
    		    $target = $user . '@' . $targetip;
		    $target_short = $targetip;
		}
		# Connect to remote storage:
		no warnings 'Net::SFTP::Foreign';
		$sftp = Net::SFTP::Foreign->new($target, password => $pass, port => $port, autodie => 0, stderr_fh => $ssherr);
		$sftp->die_on_error("Unable to connect to remote host");
		&debug_msg("INFO: Connected to $target at port $port \n");
        	# Is this a directory with the vzdump config directory in /etc/?
        	$fh1 = $sftp->stat("$Backup->{'storage'}/etc/vzdump/vps.conf");
        	if ($fh1) {
        	    &debug_msg("INFO: File $Backup->{'storage'}/etc/vzdump/vps.conf exists on remote storage. \n");
		    # OpenVZ container already present at this VPSID? If so, stop and destroy it:
    		    if (($Backup->{'virttype'} eq "openvz") && (-d "/vz/private/$Backup->{'targetvpsid'}")) {
        		&debug_msg("INFO: Stopping old VPS $Backup->{'targetvpsid'} in preparation for the restore. \n");
        		system("/usr/sbin/vzctl stop $Backup->{'targetvpsid'}");
        		# This is a bit controversial: Do we destroy the VPS? If yes, the restore may be slower. So no, we don't for now.
        	        #&debug_msg("INFO: Destroying old VPS $Backup->{'targetvpsid'} in preparation for the restore. \n");
        	        #system("/usr/sbin/vzctl destroy $Backup->{'targetvpsid'}");
    		    }
        	    # Copy the data from the remote backup location to the production environment:
        	    &debug_msg("INFO: Starting remote RSYNC from $target_short:$Backup->{'storage'}/ to /vz/private/$Backup->{'targetvpsid'}/ to restore VPS $Backup->{'targetvpsid'} \n");
        	    system("/usr/bin/rsync -q -ar -e 'ssh -p $port' $target_short:$Backup->{'storage'}/ /vz/private/$Backup->{'targetvpsid'}/ --delete-after");
        	    &debug_msg("INFO: Copying VPS config from $target_short:$Backup->{'storage'}/etc/vzdump/vps.conf to /etc/vz/conf/$Backup->{'targetvpsid'}.conf \n");
        	    system("/usr/bin/rsync -q -ar -e 'ssh -p $port' $target_short:$Backup->{'storage'}/etc/vzdump/vps.conf /etc/vz/conf/$Backup->{'targetvpsid'}.conf --delete-after");
		    system("/bin/chmod 644 /etc/vz/conf/$Backup->{'targetvpsid'}.conf");
        	    # Start the VPS again:
        	    &debug_msg("INFO: Restore of VPS $Backup->{'targetvpsid'} finished. Starting the restored VPS now. \n");
        	    system("/usr/sbin/vzctl start $Backup->{'targetvpsid'}");
        	}
	    }
	}
	# Handle remote restore of KVM's:
	if ($Backup->{'virttype'} eq "kvm") {

	    &debug_msg("INFO: OID $event->{'OID'} containing a remote KVM backup ($vpsid) at $Backup->{'storage'} is going to be restored. \n");
	    if ($user && $pass && $port && ($targethost||$targetip) && $remote_location_vzdump) {
		if ($targethost) {
    		    $target = $user . '@' . $targethost;
		    $target_short = $targethost;
		}
		else {   
    		    $target = $user . '@' . $targetip;
		    $target_short = $targetip;
		}
		# Connect to remote storage:
		no warnings 'Net::SFTP::Foreign';
		$sftp = Net::SFTP::Foreign->new($target, password => $pass, port => $port, autodie => 0, stderr_fh => $ssherr);
		$sftp->die_on_error("Unable to connect to remote host");
		&debug_msg("INFO: Connected to $target at port $port \n");
        	# Is this a directory with a virtual disk image and an XML config in it?
        	$fh1 = $sftp->stat("$Backup->{'storage'}/$vpsid.img");
        	$fh2 = $sftp->stat("$Backup->{'storage'}/$vpsid.xml");
        	if (($fh1) && ($fh2)) {
        	    &debug_msg("INFO: Disk image $Backup->{'storage'}/$vpsid.img exists on remote storage. \n");
        	    &debug_msg("INFO: XML config $Backup->{'storage'}/$vpsid.xml exists on remote storage. \n");
		    &debug_msg("INFO: Found $Backup->{'storage'} containing a remote KVM ($vpsid) rsync backup and its config file. \n");

		    # KVM VPS already present at this VPSID? If so, stop and destroy it:
    		    if (($Backup->{'virttype'} eq "kvm") && (-f "/vz/kvm/images/$Backup->{'targetvpsid'}.img")) {
        		&debug_msg("INFO: Stopping old KVM $Backup->{'targetvpsid'} in preparation for the restore. \n");
        	        system("/usr/bin/virsh destroy $Backup->{'targetvpsid'}");
    			&debug_msg("INFO: Destroying old KVM $Backup->{'targetvpsid'} in preparation for the restore. \n");
        		system("/usr/bin/virsh undefine $Backup->{'targetvpsid'}");
    		    }
## axt
		    # Handle restores:
		    if ($user eq "root") {
			# Use RSYNC for the restore as we run as 'root':
        		# Copy the data from the remote backup location to the production environment:
        		&debug_msg("INFO: Starting remote RSYNC from $target_short:$Backup->{'storage'}/$vpsid.img to /vz/kvm/images/$vpsid.img to restore KVM $Backup->{'targetvpsid'} \n");
        		system("/usr/bin/rsync -q -ar -e 'ssh -p $port' $target_short:$Backup->{'storage'}/$vpsid.img /vz/kvm/images/$vpsid.img --delete-after");
        		&debug_msg("INFO: Copying KVM config from $target_short:$Backup->{'storage'}/$vpsid.xml to /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml \n");
        		system("/usr/bin/rsync -q -ar -e 'ssh -p $port' $target_short:$Backup->{'storage'}/$vpsid.xml /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml --delete-after");
        		system("chown qemu:qemu /vz/kvm/images/$Backup->{'targetvpsid'}.img");
			system("/bin/chmod 600 /vz/kvm/images/$Backup->{'targetvpsid'}.img");
			system("/bin/chmod 600 /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml");
			system("chown root:root /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml");
			# Define the KVM again:
        		&debug_msg("INFO: Defining KVM $Backup->{'targetvpsid'} from /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml \n");
        		system("/usr/bin/virsh define /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml");
        		# Start the KVM again:
        		&debug_msg("INFO: Restore of KVM $Backup->{'targetvpsid'} finished. Starting the restored KVM now. \n");
        		system("/usr/bin/virsh start $Backup->{'targetvpsid'}");
        	    }
        	    else {
        		# Use Net::SFTP::Foreign for the restore as we don't run as root:
			# Retrieve the remote backup file via SFTP
        		&debug_msg("INFO: Starting SFTP transfer from $target_short:$Backup->{'storage'}/$vpsid.img to /vz/kvm/images/$vpsid.img to restore KVM $Backup->{'targetvpsid'} \n");
			$fh3 = $sftp->get("$Backup->{'storage'}/$vpsid.img", "/vz/kvm/images/$vpsid.img");
        		&debug_msg("INFO: Starting SFTP transfer from $target_short:$Backup->{'storage'}/$vpsid.xml to /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml to restore KVM $Backup->{'targetvpsid'} \n");
			$fh4 = $sftp->get("$Backup->{'storage'}/$vpsid.xml", "/etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml");
            		# Start restore after file retrieval:
        		if (($fh3) && ($fh4)) {
        		    system("chown qemu:qemu /vz/kvm/images/$Backup->{'targetvpsid'}.img");
			    system("/bin/chmod 600 /vz/kvm/images/$Backup->{'targetvpsid'}.img");
			    system("/bin/chmod 600 /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml");
			    system("chown root:root /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml");
			    # Define the KVM again:
        		    &debug_msg("INFO: Defining KVM $Backup->{'targetvpsid'} from /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml \n");
        		    system("/usr/bin/virsh define /etc/libvirt/qemu/$Backup->{'targetvpsid'}.xml");
        		    # Start the KVM again:
        		    &debug_msg("INFO: Restore of KVM $Backup->{'targetvpsid'} finished. Starting the restored KVM now. \n");
        		    system("/usr/bin/virsh start $Backup->{'targetvpsid'}");
        		}
        	    }
        	}
	    }
	}
	else {
	    # This is not supposed to happen:
	    &debug_msg("WARN: OID $event->{'OID'} containing an *unknown* type of remote VPS backup ($vpsid) at $Backup->{'storage'} is NOT going to be restored. \n");
	}
    }
}

$cce->bye('SUCCESS');
exit 0;

# Debugger:
sub debug_msg {
    if ($DEBUG) {
        my $msg = shift;
        $user = $ENV{'USER'};
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
}

