#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: create.pl,v 2.0.0-1Mon 21 Nov 2011 10:38:37 PM CET mstauber Exp $
# Copyright 2006-2011 Stauber Multimedia Design. All rights reserved.
#
# Handler which creates new KVM VPS's.
#

use CCE;
use POSIX qw(isalpha);

# Debugging switch:
$DEBUG = "0";
if ($DEBUG) {
    use Sys::Syslog qw( :DEFAULT setlogsock);
}

my $cce = new CCE('Domain' => 'base-vserver');

$cce->connectfd();

# Double check if the IP is not already assigned - kinda redundant, but never mind:

$vserver = $cce->event_object();
$vserver_new = $cce->event_new();
$vserver_old = $cce->event_old();

@oids= $cce->find("KVM");
($ok, $system) = $cce->get($oids[0]);

$vpsname = $vserver_new->{vpsname};

&debug_msg("INFO: controlAction: $vserver_new->{controlAction} \n");
&debug_msg("INFO: vpsname: $vserver_new->{vpsname} \n");
&debug_msg("INFO: ip: $vserver_new->{ip} \n");
&debug_msg("INFO: hostname: $vserver_new->{hostname} \n");
&debug_msg("INFO: autostart: $vserver_new->{autostart} \n");
&debug_msg("INFO: arch: $vserver_new->{arch} \n");
&debug_msg("INFO: os_type: $vserver_new->{os_type} \n");
&debug_msg("INFO: os_variant: $vserver_new->{os_variant} \n");
&debug_msg("INFO: iso_source: $vserver_new->{iso_source} \n");
&debug_msg("INFO: owner: $vserver_new->{name} \n");
&debug_msg("INFO: vcpus: $vserver_new->{vcpus} \n");
&debug_msg("INFO: memory: $vserver_new->{memory} \n");
&debug_msg("INFO: disk: $vserver_new->{disk} \n");
&debug_msg("INFO: disk_device: $vserver_new->{disk_device} \n");
&debug_msg("INFO: network: $vserver_new->{network} \n");
&debug_msg("INFO: hvm: $vserver_new->{hvm} \n");
&debug_msg("INFO: virt_type: $vserver_new->{virt_type} \n");
&debug_msg("INFO: disk_bus: $vserver_new->{disk_bus} \n");
&debug_msg("INFO: disk_size: $vserver_new->{disk_size} \n");
&debug_msg("INFO: graphics: $vserver_new->{graphics} \n");
&debug_msg("INFO: vnc_listen: $vserver_new->{vnc_listen} \n");
&debug_msg("INFO: vnc_pass: $vserver_new->{vnc_pass} \n");

# Make sure the KVM images directory is present:
if (-d "/vz/kvm/images") {
    system("mkdir -p /vz/kvm/images");
}

if (($vserver_new->{controlAction} == "create") && ($vpsname) && ($vserver_new->{iso_source})) {

    if ($vserver_new->{autostart} == "1") {
	$start = "--autostart";
    }

    &debug_msg("INFO: /usr/sbin/virt-install --hvm --name $vserver_new->{vpsname} --ram $vserver_new->{memory} --virt-type $vserver_new->{virt_type} --disk path=$vserver_new->{disk},size=$vserver_new->{disk_size} --arch $vserver_new->{arch} --vcpus $vserver_new->{vcpus} --os-type=$vserver_new->{os_type} --os-variant=$vserver_new->{os_variant} --network bridge=br0 --graphics $vserver_new->{graphics},password=$vserver_new->{vnc_pass},listen=$vserver_new->{vnc_listen} --accelerate --cdrom $vserver_new->{iso_source} $start --watchdog default \n");
    system("/usr/sbin/virt-install --hvm --name $vserver_new->{vpsname} --ram $vserver_new->{memory} --virt-type $vserver_new->{virt_type} --disk path=$vserver_new->{disk},size=$vserver_new->{disk_size} --arch $vserver_new->{arch} --vcpus $vserver_new->{vcpus} --os-type=$vserver_new->{os_type} --os-variant=$vserver_new->{os_variant} --network bridge=br0 --graphics $vserver_new->{graphics},password=$vserver_new->{vnc_pass},listen=$vserver_new->{vnc_listen} --accelerate --cdrom $vserver_new->{iso_source} $start --watchdog default");

    # Check if this transaction went through:
    unless (-f $vserver_new->{disk}) {
	&debug_msg("INFO: KVM disk $vserver_new->{disk} not present. KVM creation failed! \n");
        $cce->bye('FAIL', "[[base-vserver.KVMfailOnCreate]]");
        exit(1);
    }
    else {
	&debug_msg("INFO: KVM disk $vserver_new->{disk} present. KVM creation suceeded! \n");
    #&cleanup;
    }
}

sub cleanup {
    # Control action may also need to be reset to 'none' once we're done - but not on delete!
    if ($vserver_new->{controlAction}) {
	($sys_oid) = $cce->find('Vserver', {'vpsid' => $vpsid});
	($ok, $sysvs) = $cce->get($sys_oid);
	($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
	    'vpsid' => $vpsid,
	    'controlAction' => 'none'
	});
    }
}

# We need to say 'SUCCESS' or CCE will bitch:
$cce->bye('SUCCESS');
exit(0);

# Debugger:
sub debug_msg {
    if ($DEBUG) {
        my $msg = shift;
        $user = $ENV{'USER'};
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
}


