#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: make_sense.pl,v 2.0.0-1 Tue 25 Oct 2011 12:40:29 AM CEST mstauber Exp $
# Copyright 2006-2011 Stauber Multimedia Design. All rights reserved.
#
# Run checks on the values passed through the GUI to make sure that they make sense.
#
# The old version of this handler was way too complicated and it didn't support (yet)
# using units on settings or using the word 'unlimited' as a valid value.

use CCE;
use POSIX qw(isalpha);

$DEBUG = 0;

$cce = new CCE;

$cce->connectfd();
#$cce->connectuds(); # only for debugging

$vserver = $cce->event_object();
$vserver_new = $cce->event_new();
$vserver_old = $cce->event_old();

@oids= $cce->find("Vserver");
($ok, $system) = $cce->get($oids[0]);

# Check if VPS IP-Address causes conflict:
if (($vserver_new->{ip}) || ($vserver->{ip}) || ($vserver_old->{ip})) {

    if ($vserver->{ip} ne '') { $validate_ip = $vserver->{ip}; }
    if ($vserver_old->{ip} ne '') { $validate_ip = $vserver_old->{ip}; }
    if ($vserver_new->{ip} ne '') { $validate_ip = $vserver_new->{ip}; }

        @oids = $cce->findx("Vserver", {},
                               {
                                'ip' => $validate_ip,
                                'is_dir_present' => '1'
                               });

        # There should be no oids found
    	# 
    	# Unfortunately both find() and findx() are stupid and do partial matches.
    	# So 127.0.0.1 also matches 127.0.0.10 :o(
    	# Therefore we have to do some extra magic:
        if (scalar(@oids) > 1) {
                $the_scalar = scalar(@oids);
                $out_msg = "";
                # Walk through all returned oids. One of them is the one we want to modify. Any extra result may
                # be a partial match.
                foreach $entry (@oids) { 
                        ($ok, $conflict) = $cce->get($entry);
                        # Check if a returned match has the same hostname. If so, it's the object we want to write to and which we then ignore.
                        # Compare with 'eq' to find out if it's a fully *identical* IP:
                        if (($conflict->{ip} eq $validate_ip) && ($vserver->{hostname} ne $conflict->{hostname})) {
				$out_msg = $out_msg . $entry . " [" . $conflict->{ip} . " == " . $validate_ip . "] ";
                                # Ok, returned result is identical AND found in a VPS with differrent hostname. 
                                # So we - rightfully - throw an error:
                                $cce->bye('FAIL',
                                        "[[base-vserver.ipInUse,ip='$validate_ip (CCE Object ID: $out_msg - $the_scalar)']]");

                                exit(1);
                        }
                }
        }

        # verify that the VPS doesn't have the same IP as the master node itself:
        my ($network) = $cce->find('Network', { 'ipaddr' => $validate_ip, 'enabled' => '1' });
        if ($network) {
               	$cce->bye('FAIL',
			"[[base-vserver.ipInUseMasterNode,ip='$validate_ip (CCE Object ID: $network)']]");
                exit(1);
        }

        # Also check vpsIPalloc for IPs that may be assigned as 2ndary IPs to other VPSs:
        @oids = $cce->findx("vpsIPalloc",
                               {
                                'min' => $validate_ip
                               });

        # there should be no oids found
        if (scalar(@oids) > 0) {
                $the_scalar = scalar(@oids);
                foreach $line (@oids) {
                        $out_msg = $out_msg . " " . $line;
                }

                $cce->bye('FAIL',
                          "[[base-vserver.ipInUse,ip='$validate_ip (CCE Object ID:$out_msg - $the_scalar)']]");
                exit(1);
        }
}

# hostname must be less than or equal to 255
if ((length($vserver->{hostname}) > 255) ||
    (length("$vserver->{hostname}") > 255)) {

    $cce->bye('FAIL', "[[base-vserver.hostnameTooLong]]");
    exit(1);
}

# CPU-Units check:
$cpuunits = $vserver_new->{cpuunits};
$cpuunits_old = $vserver->{cpuunits};
$cpuunits_node = $vserver->{cpuunits_node};
$cpuunits_alloc = $vserver->{cpuunits_alloc};

#if (($cpuunits_node) && ($cpuunits > $cpuunits_node)) { # $cpuunits_node may not be set yet on first run of constructor
#    $cce->bye('FAIL', "[[base-vserver.cpuunitsnode]]");
#        exit(1);
#}    

#if (($cpuunits_node) && ($cpuunits > $cpuunits_node-$cpuunits_alloc)) { # $cpuunits_node may not be set yet on first run of constructor
#    $cce->bye('FAIL', "[[base-vserver.cpuunitsnodeavail]]");
#        exit(1);
#}    
#
### ^- Messer =!= Sau

## Check selected VPS parameters to see if they are within acceptable ranges:

# Hash of VPS parameters with their names and info about how barrier and limit are named and also the info if kilo means '1000' or '1024".
# "K"  = 1000
# "KB" = 1024

%vpsParams = (
  "physpages", "physpages:b:l:KB",
  "swappages", "swappages:b:l:KB",
  "kmemsize", "kmemsize:b:l:KB",
  "dcachesize", "dcachesize:b:l:KB",
  "lockedpages", "lockedpages:b:l:KB",
  "diskspace", "diskspace:s:h:KB",
  "vmguarpages", "vmguarpages:b:l:KB",
  "shmpages", "shmpages:b:l:KB",
  "privvmpages", "privvmpages:b:l:KB",
  "oomguarpages", "oomguarpages:b:l:K",
  "avnumproc", "avnumproc:b:l:K",
  "numproc", "numproc:b:l:K",
  "numtcpsock", "numtcpsock:b:l:K",
  "numothersock", "numothersock:b:l:K",
  "tcpsndbuf", "tcpsndbuf:b:l:K",
  "tcprcvbuf", "tcprcvbuf:b:l:K",
  "othersockbuf", "othersockbuf:b:l:K",
  "dgramrcvbuf", "dgramrcvbuf:b:l:K",
  "numfile", "numfile:b:l:K",
  "numflock", "numflock:b:l:K",
  "numpty", "numpty:b:l:K",
  "numsiginfo", "numsiginfo:b:l:K",
  "numiptent", "numiptent:b:l:K",
  "diskinodes", "diskinodes:s:h:K"
);

# Walk through the hash one by one:
foreach $value (values %vpsParams) {

    # Split the values at the ':' delimiter:
    @workparm = split(/:/, $value);

    # Get the info we need and assemble it into variables:
    $work_parm = $workparm[0];
    $work_barrier = $workparm[0]."_".$workparm[1];
    $work_limit = $workparm[0]."_".$workparm[2];
    $work_unit = $workparm[3];

    # Run the validity checks:
    &settings_check($work_parm, $work_barrier, $work_limit, $work_unit);
}

######

sub settings_check {

  # This subroutine takes four arguments:
  # $parameter:		name of the parameter. Like 'numproc' or 'avnumproc'.
  # $barrier:		name of the barrier. Like 'numproc_b' or 'avnumproc_b'.
  # $limit:		name of the limit. Like 'numproc_l' or 'avnumproc_l'.
  # $unit:		Either 'K' for a factor of 1000. Or 'KB' for a factor of 1024.
  #
  # Purpose of this subroutine is to check which value (barrier or limit) is new and to
  # then run checks if barrier or limit are either equal or if barrier is smaller than 
  # limit. Also allowed: Both set to 'unlimited'. Or barrier set to a number and limit
  # set to 'unlimited'.
  #
  # Note: We use the subroutine 'compare_human_readable' for these checks.
  #
  # If the checks don't find any objectionable settings, no error is raised.
  # If barrier is bigger than limit, then a CCE error is raised.

  my ($parameter, $barrier, $limit, $unit) = @_;


  if (($vserver_new->{$barrier}) || ($vserver_new->{$limit})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{$barrier} > 0) {
        $bar_new = 1;
    }
    else {
        $bar_new = 0;
    }
    if ($vserver_old->{$barrier} > 0) {
        $bar_old = 1;
    }
    else {
        $bar_old = 0;
    }
    if ($vserver_new->{$limit} > 0) {
        $lim_new = 1;
    }
    else {
        $lim_new = 0;
    }
    if ($vserver_old->{$limit} > 0) {
        $lim_old = 1;
    }
    else {
        $lim_old = 0;
    }

    $error_msg_param = '[[base-vserver.' . $parameter . '_mismatch]]';
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && (&compare_human_readable($vserver_new->{$barrier}, $vserver_old->{$limit}, $unit) == "1")) {

            $cce->bye('FAIL', $error_msg_param);
            exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && (&compare_human_readable($vserver_new->{$barrier}, $vserver_new->{$limit}, $unit) == "1")) {
            $cce->bye('FAIL', $error_msg_param);
            exit(1); 
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && (&compare_human_readable($vserver_old->{$barrier}, $vserver_new->{$limit}, $unit) == "1")) {
            $cce->bye('FAIL', $error_msg_param);
            exit(1);
    }
  }
}

######

sub compare_human_readable {
     
    # This subroutine takes three arguments:
    #
    # barrier		The barrier value
    # limit		The limit value
    # unit		The unit ('K' or 'KB') which defines the multiplicator.

    # It checks if they are in human readable format with unit, or pure integers.
    # It turns human readable numbers into their native interger format to be
    # able to really compare them. Otherwise '2K' would be smaller than '1999'.

    # Possible return values:
    #
    #	0 = OK 		limit higher than barrier, or at least equal to it
    #	1 = ERROR	barrier higher than limit, which must NOT be the case

    my ($barrier, $limit, $unit) = @_;
    
    #
    ### Barrier checks:
    #

    # Start sane:
    $barrier_is_numerical = "0";
    
    # Check for integer:
    if ($barrier =~ /^\d+$/) {
	$barrier_is_numerical = "1";
    }
    
    # Check for positive decimal number with unit of format 'K', 'M', 'G' or 'T':
    if ($barrier =~ /^(\d*\.{0,1}\d+)(K|M|G|T)$/) {
	$value = $1;
	$val_unit = $2;

	if ($unit eq "K") { $multi = "1000"; }
	if ($unit eq "KB") { $multi = "1024"; }

	if ($val_unit eq "K") {
	    $mod = $multi;
	    $barrier = $barrier*$mod;
	}
	if ($val_unit eq "M") {
	    $mod = $multi*$multi;
	    $barrier = $barrier*$mod;
	}
	if ($val_unit eq "G") {
	    $mod = $multi*$multi*$multi;
	    $barrier = $barrier*$mod;
	}
	if ($val_unit eq "T") {
    	    $mod = $multi*$multi*$multi*$multi;
	    $barrier = $barrier*$mod;
	}
	$barrier_is_numerical = "1";
    }
        
    # Check for positive decimal number without unit:
    if ($barrier =~ /^(\d*\.{1}\d+)$/) {
	$barrier_is_numerical = "1";
    }   
        
    # Check for the word 'unlimited' (case insensitive)
    if ($barrier =~ /^unlimited$/i) {
	$barrier_is_numerical = "0";
    }

    #
    ### Limit checks:
    #

    # Start sane:
    $limit_is_numerical = "0";
    
    # Check for integer:
    if ($limit =~ /^\d+$/) {
	$limit_is_numerical = "1";
    }
    
    # Check for positive decimal number with unit of format 'K', 'M', 'G' or 'T':
    if ($limit =~ /^(\d*\.{0,1}\d+)(K|M|G|T)$/) {
	$value = $1;
	$val_unit = $2;

	if ($unit eq "K") { $multi = "1000"; }
	if ($unit eq "KB") { $multi = "1024"; }

	if ($val_unit eq "K") {
	    $mod = $multi;
	    $limit = $limit*$mod;
	}
	if ($val_unit eq "M") {
    	    $mod = $multi*$multi;
	    $limit = $limit*$mod;
	}
	if ($val_unit eq "G") {
	    $mod = $multi*$multi*$multi;
	    $limit = $limit*$mod;
	}
	if ($val_unit eq "T") {
    	    $mod = $multi*$multi*$multi*$multi;
	    $limit = $limit*$mod;
	}
	$limit_is_numerical = "1";
    }
        
    # Check for positive decimal number without unit:
    if ($limit =~ /^(\d*\.{1}\d+)$/) {
	$limit_is_numerical = "1";
    }   
        
    # Check for the word 'unlimited' (case insensitive)
    if ($limit =~ /^unlimited$/i) {
	$limit_is_numerical = "0";
    }

    #
    ### Now do the actual comparation of barrier and limit:
    #

    if (($barrier_is_numerical == "0") && ($limit_is_numerical == "0")) {
        # As both are not numerical, they must be both unlimited.
        # Check anyway:
        if ($barrier eq $limit) {
	    # Both unlimited - return 0
	    return 0;
	}
	else {
	    # One unlimited, one not - return 1
	    return 1;
	}
    }
    if (($barrier_is_numerical == "0") && ($limit_is_numerical == "1")) {
	# Barrier is a number, Limit is unlimited - return 0
	return 0;
    }

    if (($barrier_is_numerical == "1") && ($limit_is_numerical == "0")) {
	# Barrier is unlimited, Limit is a number - return 1
	return 1;
    }
    if (($barrier_is_numerical == "1") && ($limit_is_numerical == "1")) {
	# Both are numbers. Compare them:
	if ($barrier <= $limit) {
	    # Barrier smaller than Limit or of same size - return 0
	    return 0;
	}
	else {
	    # Barrier bigger than limit - return 1
	    return 1;
	}
    }
}

$cce->bye('SUCCESS');
exit(0);
