#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: generate_vserver_list.pl, v2.0.0-1 Fri 28 Oct 2011 06:02:26 AM CEST mstauber Exp $
# Copyright 2006-2011 Stauber Multimedia Design. All rights reserved.

# This constructor / handler uses 'vzlist' to poll extended vserver status.
# Additionally it parses the vserver config files in /etc/sysconfig/vz-scripts/
# The information is then written off into CCE for GUI usage. 

use CCE;
my $cce = new CCE;
my $conf = '/var/lib/cobalt';

use Sys::Virt;
use Sys::Virt::Domain;
use XML::Simple;
use Data::Dumper;

$cce->connectfd();
#$cce->connectuds(); # only for debugging

# Location of the vzlist-dump:
$vzlistdump = '/tmp/vserver-list-short';
$vzcfgdir = '/etc/sysconfig/vz-scripts/';

# Debugging switch:
$DEBUG = "0";

# Location of libvirt config files (directory - with trailing slash):
$cfgdir = '/etc/libvirt/qemu/';

# Generate vzlist-dump:
system("/usr/sbin/vzlist -a -o vpsid,ip,hostname,status,numproc,laverage,cpuunits,hostname | /bin/sed 's/^[ \t]*//;s/[ \t]*\$//' > $vzlistdump");

# Pull the vzlist-dump into a Matrix:
#
# I LOVE matrix stuff! If there is a sledload of data to parse, 
# then there is hardly a more efficient way to do so.
# A legend of what colums the matrix has is at the end of the code.

    # Array of VPSIDs:
    @matrix_vpsids = ();

    open (F, $vzlistdump) || die "Could not open $vzlistdump: $!";
    $a = 0;                                     # Matrix line number, whereas 0 is the first (useable) line of the vzlist dump
    while ($line = <F>) {

        chomp($line);
        next if $line =~ /^\s*$/;               # skip blank lines
        next if $line =~ /^VEID/;               # skip lines starting with VEID
        next if $line =~ /^CTID/;               # skip lines starting with VEID
        $matrix_name = 'VPSID';
        my (@row) = split (/\s+/, $line);
        push (@{$matrix_name}, \@row);

        # Build an array that contains the VPSIDs:
        if ($VPSID[$a][0]) {                    # Plausibility check. We will not store anything into CCE if 'vpsid' is not set.
            $currentvpsid = $VPSID[$a][0];
            push (@matrix_vpsids, $VPSID[$a][0]);
        }

	# Fix for ultra-long hostnames:
	$VPSID[$a][2] = $VPSID[$a][7];

        # Check if VPS private area exists. Because it may not exist anymore, as the VPS could have been migrated off or manually have been deleted:
        $vps_home_area = "/vz/private/" . $VPSID[$a][0];
        if (-d $vps_home_area ) {
            $VPSID[$a][126] = "1";
        }
        else {
            $VPSID[$a][126] = "0";
        }

        $a++;
    }
    close(F);
    system("rm $vzlistdump");

    # At this point we have all data from VZLIST in a matrix. But there is more data to be had in the config files. We need to get that as well:

    # How many entries do we have in the Matrix?
    $matrix_entries = $#VPSID;

    # How many VPSIDs are in the Array?
    $num_matrix_vpsids = $#matrix_vpsids;

    # Set start of Matrix line:
    $matrix_line = 0;

    foreach $currentvpsid (@matrix_vpsids) {
        &feedthemonster;                        # Feed info into CCE
        $matrix_line++;
    }

&cpu_alloc;
&kvm;

# Subroutine that feeds the data into CCE:
sub feedthemonster {
	&readconfig;
	&validate;

        # Check if the VPS is already present in CCE under the old VPS ID:
        @oids = $cce->find('Vserver', {'vpsid' => $currentvpsid});

        # Check if the VPS is already present under a different VPS ID:
        @moved_oids = $cce->find('Vserver', {'ip' => $VPSID[$matrix_line][1], 'hostname' => $VPSID[$matrix_line][7]});

        # Legend:
        # $#oids : $#moved_oids : action
        #
        #  0 : 0  : update
        # -1 : 0  : update
        # -1 : -1 : create:

        if (($#oids == -1) && ($#moved_oids == -1)) {
                #print "Object not yet in CCE. Creating new one.\n";
                ($ok) = $cce->create('Vserver', {
                'vpsid' => $VPSID[$matrix_line][0],
                'ip' => $VPSID[$matrix_line][1],
                'hostname' => $VPSID[$matrix_line][2],
		'ONBOOT' => $VPSID[$matrix_line][114],
#		'name' => 'admin',
                'controlAction' => 'none',
                'timestamp' => time()
                });
		$new_vps_found = '1'; 			# We found a new VPS, one that we have never seen before and which
							# is not yet in CCE. We do not want to risk to populate CCE with ALL
							# the found new data in one go. So let us just set the most basic info
							# and the timestamp. 
							#
							# We also set $new_vps_found = '1' to run feedthemonster() again.
        }
        else {
	        #print "Object already present in CCE. Updating it.\n";
                # find the system object
		$new_vps_found = '0';
		&validate;

                if (($#oids < 0) && ($#moved_oids == -1)) {
                        ($sys_oid) = $cce->find('Vserver', {'vpsid' => $currentvpsid});
                        ($ok, $sys) = $cce->get($sys_oid);
                }
                else {
                        ($sys_oid) = $cce->find('Vserver', {'ip' => $VPSID[$matrix_line][1], 'hostname' => $VPSID[$matrix_line][2]});
                        ($ok, $sys) = $cce->get($sys_oid);
                }

		# For debugging and logging purpose we do not write all 122 entries into CCE in one go. Instead we break it down
		# into multiple blocks so that we can troubleshoot /var/log/messages easily for data-blocks with values that CCE
		# did not like. 
		
                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'vpsid' => $VPSID[$matrix_line][0],
                'ip' => $VPSID[$matrix_line][1],
                'hostname' => $VPSID[$matrix_line][2],
                'status' => $VPSID[$matrix_line][3],
                'numproc' => $VPSID[$matrix_line][4],
                'laverage' => $VPSID[$matrix_line][5],
                'cpuunits' => $VPSID[$matrix_line][6],
                'ONBOOT' => $VPSID[$matrix_line][114],
#                'name' => 'admin',
                'controlAction' => 'none',
                'is_dir_present' => $VPSID[$matrix_line][126],
                'timestamp' => time()
                });
	}

}


# Validate the settings before we pass them on to CCE:
sub validate {

    # ip:
    if ($VPSID[$matrix_line][1] == 0) {
	$VPSID[$matrix_line][1] = "127.0.0.1";
    }
    # hostname:
    if (!$VPSID[$matrix_line][2]) {
	$VPSID[$matrix_line][2] = "localhost.localdomain";
    }

    # numproc:
    if ($VPSID[$matrix_line][4] eq "-") {
	$VPSID[$matrix_line][4] = "0";
    }
    # laverage:
    if ($VPSID[$matrix_line][5] eq "-") {
	$VPSID[$matrix_line][5] = "0";
    }
}

sub readconfig {

    # We need to parse the config file for the onboot switch:
    $cfile = "$vzcfgdir$currentvpsid.conf";
    if (-e $cfile) {
        open(CFILE,"$cfile") || die("Could not find $cfile. $!");
        while ($line = <CFILE>) {
            chomp($line);
            next if $line =~ /^\s*$/; # skip blank lines

            if ($line =~ /^ONBOOT=\"([A-Za-z]\w*)\"/) {
                $onboot = $1;
                if (($onboot eq 'yes') || ($onboot eq 'Yes')) {
                    $VPSID[$matrix_line][114] = 'Yes';
                }
                else {
                    $VPSID[$matrix_line][114] = 'No';
                }
	    }
	}
    }
}

sub cpu_alloc {

    ##
    # Get info about the current CPU utilization and the power of the node:
    ##

    # Location of the vzcpu-dump:
    $vzcpudump = '/tmp/vserver-cpuinfo';

    # Generate vzcpu-dump:
    system("/usr/sbin/vzcpucheck | /bin/sed 's/^[ \t]*//;s/[ \t]*\$//' > $vzcpudump");

    if (-e $vzcpudump) {
        $b = 0;
        open(CPUDUMP,"$vzcpudump") || die("Could not find $vzcpudump. $!");
        while ($line = <CPUDUMP>) {
            chomp($line);
            next if $line =~ /^\s*$/; # skip blank lines
            if (($line =~ /^Current CPU utilization: (.*)/) && ($b == 0)) {
                $CURRENT_CPU = $1;
            }
            if (($line =~ /^Power of the node: (.*)/) && ($b == 1)) {
                $NODE_CPU = $1;
            }
            $b++;
        }
        close(CPUDUMP);
        system("/bin/rm $vzcpudump");
        &cce_update_cpu; # Write the info into CCE
    }
}

sub cce_update_cpu {

	##
	# Update or populate CCE with the CPU_POWER and CURRENT_CPU (usage) of the master node:
	##

        # This is where we actually populate CCE with the gathered info:
        unless (($CURRENT_CPU == "") || ($NODE_CPU == "")) {
    	    @oids = $cce->find('VserverMasterNode');
    	    if ($#oids < 0) {
                #print "Object not yet in CCE. Creating new one.\n";
                ($ok) = $cce->create('VserverMasterNode', {
                'CURRENT_CPU' => $CURRENT_CPU,
                'NODE_CPU' => $NODE_CPU
                });
    	    }
    	    else {
                # find the system object
                ($sys_oid) = $cce->find('VserverMasterNode');
                ($ok, $sys) = $cce->get($sys_oid);
                ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'CURRENT_CPU' => $CURRENT_CPU,
                'NODE_CPU' => $NODE_CPU
                });
    	    }
    	}
}

sub kvm {

    # Check which KVM's are pesent (if any):
    $hv = Sys::Virt->new(address => "", readonly => 1);

    $interval = @ARGV ? shift @ARGV : 1;
    $iterations = @ARGV ? shift @ARGV : 1;

    %states = (
        &Sys::Virt::Domain::STATE_NOSTATE => "nostate",
        &Sys::Virt::Domain::STATE_RUNNING => "running",
        &Sys::Virt::Domain::STATE_BLOCKED => "blocked",
        &Sys::Virt::Domain::STATE_PAUSED => "paused",
        &Sys::Virt::Domain::STATE_SHUTDOWN => "shutdown",
        &Sys::Virt::Domain::STATE_SHUTOFF => "shutoff",
        &Sys::Virt::Domain::STATE_CRASHED => "crashed",
        &Sys::Virt::Domain::STATE_RUNNING => "running",
    );

    %cpuTime;
    $sample;

    for ($i = 0 ; $i < $iterations ; $i++) {
        sleep $interval if $i;
        $now = time;   

        @domains_running = $hv->list_domains();
        @domains_stopped = $hv->list_defined_domains();
        @domains = (@domains_running, @domains_stopped);

        @stats;
        foreach $domain (sort { $a->get_id <=> $b->get_id } @domains) {
            $uuid = $domain->get_uuid_string;

            $info = $domain->get_info;

            $cpudelta = exists $cpuTime{$uuid} ? $info->{cpuTime} - $cpuTime{$uuid} : 0;
            $timedelta = defined $sample ? ($now - $sample)*1000*1000*1000 :0;

            $cpuTime{$uuid} = $info->{cpuTime};

            $util = $timedelta > 0 ? $cpudelta * 100 / $timedelta : 0;

            if ($domain->get_autostart() == "-1") {
                $autostart = "0";
            }
            else {
                $autostart = "1";
            }
            if ($domain->get_os_type() == "hvm") {
                $hvm = "1";
            }
            else {
                $hvm = "0";
            }

            # Build Matrix:
            $vpsid = $domain->get_name;
            if ($KVMID[$vpsid][0] ne "-1") {
                $KVMID[$vpsid][0] = $domain->get_id;
            }
            else {
                $KVMID[$vpsid][0] = "0";
            }
            $KVMID[$vpsid][1] = $domain->get_name;
            $KVMID[$vpsid][25] = $states{$info->{state}};
            $KVMID[$vpsid][28] = (sprintf "%d%%",$util);
            $KVMID[$vpsid][3] = $info->{memory};
            $KVMID[$vpsid][2] = $info->{nrVirtCpu};
            $KVMID[$vpsid][29] = $info->{maxMem};
            $KVMID[$vpsid][30] = $autostart;
            $KVMID[$vpsid][14] = $hvm;

            # Get the rest from XML:
            $vpsxml = $domain->get_xml_description();
            $xml = new XML::Simple;
            $data = $xml->XMLin("$vpsxml");

            $KVMID[$vpsid][4] = $data->{'os'}->{'type'}->{'arch'};
            $KVMID[$vpsid][8] = $data->{'os'}->{'type'}->{'machine'};
            $KVMID[$vpsid][31] = $data->{'devices'}->{'disk'}->[0]->{'source'}->{'file'};
            $KVMID[$vpsid][10] = $data->{'devices'}->{'disk'}->[0]->{'target'}->{'bus'};
            $KVMID[$vpsid][9] = $data->{'devices'}->{'disk'}->[0]->{'device'};
            $KVMID[$vpsid][19] = $data->{'devices'}->{'graphics'}->{'type'};
            $KVMID[$vpsid][21] = $data->{'devices'}->{'graphics'}->{'listen'}[0];
            $KVMID[$vpsid][22] = $data->{'devices'}->{'graphics'}->{'port'};
            $disk = $domain->get_block_info($KVMID[$vpsid][31]);
            $KVMID[$vpsid][11] = $disk->{'capacity'};
            $KVMID[$vpsid][32] = $disk->{'allocation'};
            $KVMID[$vpsid][12] = $data->{'devices'}->{'interface'}->{'type'};
            $KVMID[$vpsid][15] = $data->{'type'};
            $KVMID[$vpsid][18] = $KVMID[$vpsid][30]; # hmpf ... have this twice.

            # VNC passwd cannot be obtained through $domain->get_xml_description();
            # For that we parse the XML configs in '/etc/libvirt/qemu/', which we'd probably do right from the start instead.
    
            # create object
            $vncxml = new XML::Simple(ForceArray => 1, KeepRoot => 1, NoSort => 0);
    
            # read XML file
            $vncdata = $vncxml->XMLin("$cfgdir$vpsid.xml");
            $VPSID[$vpsid][20] = $vncdata->{domain}->[0]->{'devices'}->[0]->{'graphics'}->[0]->{'passwd'};

            # Check if the file containing the KVM disk is present:
            if (-f $KVMID[$vpsid][31]) {
                $is_dir_present = "1";
                $KVMID[$vpsid][26] = "1";
            }
            else {
                $is_dir_present = "0";
                $KVMID[$vpsid][26] = "0";
            }
            # Check if the XML config file containing the KVM config is present:
            $the_file = $cfgdir . $KVMID[$vpsid][1] . ".xml";
            if (-f $the_file) {
                $KVMID[$vpsid][27] = "1";
            }
            else {
                $KVMID[$vpsid][27] = "0";
            }

            if ($DEBUG == "1") {
                print "vpsid " . $KVMID[$vpsid][0] . "\n";
                print "vpsname " . $KVMID[$vpsid][1] . "\n";
                print "status " . $KVMID[$vpsid][25] . "\n";
                print "cpuTime " . $KVMID[$vpsid][28] . "\n";
                print "memory " . $KVMID[$vpsid][3] . "\n";
                print "vcpus " . $KVMID[$vpsid][2] . "\n";
                print "autostart " . $KVMID[$vpsid][30] . "\n";
                print "hvm " . $KVMID[$vpsid][14] . "\n";
                print "arch " . $KVMID[$vpsid][4] . "\n";
                print "os_variant " . $KVMID[$vpsid][8] . "\n";
                print "disk " . $KVMID[$vpsid][31] . "\n";
                print "disk_device " . $KVMID[$vpsid][9] . "\n";
                print "disk_bus " . $KVMID[$vpsid][10] . "\n";
                print "disk_size " . $KVMID[$vpsid][11] . "\n";
                print "disk_alloc " . $KVMID[$vpsid][32] . "\n";
                print "graphics " . $KVMID[$vpsid][19] . "\n";
                print "vnc_listen " . $KVMID[$vpsid][21] . "\n";
                print "vnc_port " . $KVMID[$vpsid][22] . "\n";
		print "vnc_pass " . $VPSID[$vpsid][20] . "\n";
                print "is_dir_present " . $KVMID[$vpsid][26] . "\n";
                print "is_vps_present " . $KVMID[$vpsid][27] . "\n";
                print "network " . $KVMID[$vpsid][12] . "\n";
                print "virt_type " . $KVMID[$vpsid][15] . "\n";
                print "\n";
                #print Dumper($data);
            }
            # Write the data off into CODB:
            &kvmfeedthemonster;
        }
        $sample = $now;
    }
}

# Subroutine that feeds the data into CCE:
sub kvmfeedthemonster {

        # This is where we actually populate CCE with the gathered info:
        $vpsname = $KVMID[$vpsid][1]; 
                
        # Check if the VPS is already present in CCE:
        @oids = $cce->find('KVM', {'vpsname' => $vpsname});

        if ($#oids == -1) {
                # Object not yet in CCE. Creating new one:
                #'vpsid' => $KVMID[$vpsid][0],
                ($ok) = $cce->create('KVM', {
                'vpsname' => $KVMID[$vpsid][1],
                'vcpus' => $KVMID[$vpsid][2],
                'memory' => $KVMID[$vpsid][3],
                'arch' => $KVMID[$vpsid][4],
                'os_type' => $KVMID[$vpsid][7],
                'os_variant' => $KVMID[$vpsid][8],
                'disk_device' => $KVMID[$vpsid][9],
                'disk_bus' => $KVMID[$vpsid][10],
                'disk_size' => $KVMID[$vpsid][11],
                'network' => $KVMID[$vpsid][12],
                'hvm' => $KVMID[$vpsid][14],
                'virt_type' => $KVMID[$vpsid][15],
                'autostart' => $KVMID[$vpsid][18],
                'graphics' => $KVMID[$vpsid][19],
                'vnc_listen' => $KVMID[$vpsid][21],
                'vnc_port' => $KVMID[$vpsid][22],
                'status' => $KVMID[$vpsid][25],
                'is_dir_present' => $KVMID[$vpsid][26],
                'is_vps_present' => $KVMID[$vpsid][27],
                'cpuTime' => $KVMID[$vpsid][28],
                'currMem' => $KVMID[$vpsid][29],
                'disk' => $KVMID[$vpsid][31],
                'disk_alloc' => $KVMID[$vpsid][32],
                'controlAction' => 'none',
                'timestamp' => time()
                });
        }
        else {
                # Object already present in CCE. Updating it.
                ($ok) = $cce->set($oids[0], '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'vpsname' => $KVMID[$vpsid][1],
                'vcpus' => $KVMID[$vpsid][2],
                'memory' => $KVMID[$vpsid][3],
                'arch' => $KVMID[$vpsid][4],
                'os_type' => $KVMID[$vpsid][7],
                'os_variant' => $KVMID[$vpsid][8],
                'disk_device' => $KVMID[$vpsid][9],
                'disk_bus' => $KVMID[$vpsid][10],
                'disk_size' => $KVMID[$vpsid][11],
                'network' => $KVMID[$vpsid][12],
                'hvm' => $KVMID[$vpsid][14],
                'virt_type' => $KVMID[$vpsid][15],
                'autostart' => $KVMID[$vpsid][18],
                'graphics' => $KVMID[$vpsid][19],
                'vnc_listen' => $KVMID[$vpsid][21],
                'vnc_port' => $KVMID[$vpsid][22],
                'status' => $KVMID[$vpsid][25],
                'is_dir_present' => $KVMID[$vpsid][26],
                'is_vps_present' => $KVMID[$vpsid][27],
                'cpuTime' => $KVMID[$vpsid][28],
                'currMem' => $KVMID[$vpsid][29],
                'disk' => $KVMID[$vpsid][31],
                'disk_alloc' => $KVMID[$vpsid][32],
                'controlAction' => 'none',
                'timestamp' => time()
                });

                # We may not have a 'vnc_pass' set, so we set it separately if there is one:
                if ($VPSID[$vpsid][20] ne "") {
                    ($ok) = $cce->set($oids[0], '',{    # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                    'vnc_pass' => $VPSID[$vpsid][20],
                    'timestamp' => time()
                    });
                }
            }
}

$cce->bye('SUCCESS');
exit(0);

